#!/usr/bin/env bash

# Global color definitions.
NC='\033[0m'
RED='\033[0;31m'
GREEN='\033[0;32m'


# See if arguments point to result folders with .yaml files of equal length.
function parse_args {
    local yaml_path=""
    local yaml_length=-1
    local cur_yaml_length=0
    
    for path in $@; do
        # Check if first dir exist.
        [ ! -d $path ] && { echo -e "\u274c FATAL: $path does not exist" 1>&2; return 1; }
        yaml_path=$path/result.*.yaml

        # Check if first dir has .yaml files.
        NF=$(ls -1 $yaml_path 2>/dev/null | wc -l)
        [ "$NF" != 1 ] && { echo -e "\u274c FATAL: $path does not have .yaml files" 1>&2; return 1; }
    
        # Overwrite stored .yaml file length if not stored yet.
        cur_yaml_length=$(wc -l < $yaml_path)
        [ $yaml_length == -1 ] && yaml_length=$cur_yaml_length

        # Check if .yaml file has same length.
        [ $cur_yaml_length != $yaml_length ] && { echo -e "\u274c FATAL: $path's .yaml file is of different length ($yaml_length != $cur_yaml_length)" 1>&2; return 1; }
    done

    echo -e "\u2714 Both result files exist and have .yaml files of equal length."
}

# Compare the output of the .yaml files, given as arguments.
function compare_yaml {
    local model_reached=0
    local varname
    local val1
    local val2

    # Go line by line over yaml files and print difference.
    while read line1 <&3 && read line2 <&4; do
        # Pass till model section is reached.
        if [ $model_reached == 0 ]; then
            [ "$line1" == "model:" ] && model_reached=1
            continue
        fi

        # Stop when timing section is reached.
        [ "$line1" == "timing:" ] && break 

        # Extract values from the lines.
        varname=${line1%: *}
        val1=${line1#*: }
        val2=${line2#*: }

        # Parse data type for check.
        if [ "${val1:0:1}" = "[" ]; then
            # Parse variables as arrays.
            echo "TODO"
        else
            # Parse variables as ints.
            if [ $val1 -eq $val2 ]; then
                echo -e "$val1\t\t\t\t$val2"
            else
                echo -e "$RED$val1$NC\t\t\t\t$GREEN$val2$NC"
            fi
        fi
    done 3<$1/result.*.yaml 4<$2/result.*.yaml
}

# Entry point.
function main {
    # On entry, parse given paths to results.
    path1=$1
    path2=$2
    parse_args $path1 $path2
    compare_yaml $path1 $path2
}

main $@
